unit hiRuler; {  Ruler () ver 2.30 for Delphi and FPC}

interface
uses Windows, Messages, KOL, Share, Debug, Win;

type
   TSizeUnit = ( suCentimeters, suInches, suPixels );

   THIRuler = class(THIWin)
      private
         FUp, FDown: Pbitmap;
         FFlag: Boolean;
         FUseUnit: TSizeUnit;
         LOGPIXELSX_, LOGPIXELSY_ : integer;
         Fold, FLeftPosition, FRightPosition: integer;
         procedure PaintLine;
         procedure InitRDO;
         procedure InitRUP;         
         procedure SetLeftPosition(const Value: integer);
         procedure SetRightPosition(const Value: integer);
         procedure _OnPaint( Sender: PControl; DC: HDC );
      public
         _prop_Style: byte;
         _prop_SizeUnit: byte;
         _prop_LineStyle: byte;
         _prop_LineColor: TColor;
         _prop_LeftPosition: integer;
         _prop_RightPosition: integer;
         _data_DHandle: THI_Event;
         destructor Destroy; override;
         procedure Init; override;
         procedure _work_doSizeUnit(var _Data:TData; Index:word);
         procedure _work_doRefresh(var _Data:TData; Index:word);
         procedure _work_doLeftPosition(var _Data:TData; Index:word);
         procedure _work_doLineStyle(var _Data:TData; Index:word);
         procedure _work_doLineColor(var _Data:TData; Index:word);
         procedure _work_doRightPosition(var _Data:TData; Index:word);
         procedure _var_CurPosition(var _Data:TData; Index:word);
         procedure _var_ULeftPosition(var _Data:TData; Index:word);
         procedure _var_URightPosition(var _Data:TData; Index:word);
      end;

implementation

function WndProcRuler( Sender: PControl; var Msg: TMsg; var Rslt: Integer ): Boolean;
var   HRuler: THIRuler;
begin
   Result:=False;
   HRuler:= THIRuler( Sender.Tag );
   with HRuler do
   Case Msg.Message of
      WM_LBUTTONDOWN: begin 
         if HiWord(Msg.Lparam) < Sender.Height div 2 then FFlag := true;
         _OnPaint(Sender, Sender.Canvas.Handle);
         end;
      WM_LBUTTONUP: FFlag := false;
      WM_MOUSEMOVE:
         if (Msg.wParam and MK_LBUTTON) > 0 then begin
            begin
               if FFlag then
                  SetLeftPosition(loword(Msg.lParam))
               else
                  SetRightPosition(loword(Msg.lParam));
            end;
         _OnPaint(Sender, Sender.Canvas.Handle);
         end;
   end;
end;

procedure THIRuler.Init;
begin
   Control:= NewPanel(FParent, TEdgeStyle(_prop_Style));
   Control.OnPaint:= _OnPaint;
   Control.Tag:= LongInt(Self);
   Control.AttachProc(WndProcRuler);
   FUseUnit:= TSizeUnit(_prop_SizeUnit);
   FLeftPosition:= _prop_LeftPosition;
   FRightPosition:= _prop_RightPosition;
   InitRDO;
   InitRUP;
   inherited;
   if FRightPosition >= Control.Width then FRightPosition := Control.Width - 6; 
end;

destructor THIRuler.Destroy;
begin
   FUp.free;
   FDown.free;
   inherited Destroy;
end;

procedure THIRuler._work_doSizeUnit;
begin
   _prop_SizeUnit:= ToInteger(_Data);
   FUseUnit:= TSizeUnit(_prop_SizeUnit);
   Control.Invalidate;
end;

procedure THIRuler._work_doLeftPosition;
begin
   _prop_LeftPosition:= ToInteger(_Data);
   FLeftPosition:= _prop_LeftPosition;
   Control.Invalidate;
end;

procedure THIRuler._work_doRightPosition;
begin
   _prop_RightPosition:= ToInteger(_Data);
   FLeftPosition:= _prop_RightPosition;
   Control.Invalidate;
end;

procedure THIRuler._work_doRefresh;
begin
   Control.Invalidate;
end;

procedure THIRuler._work_doLineStyle;
begin
   _prop_LineStyle:= ToInteger(_Data);
   PaintLine;
end;

procedure THIRuler._work_doLineColor;
begin
   _prop_LineColor:= ToInteger(_Data);
   PaintLine;
end;

procedure THIRuler._OnPaint;
const Offset: array[boolean] of integer = ( 4, 6 );
var   x, y: single;
      pt: TPoint;
      str: string;
      R,M: TRect;
      Wnd: HWND;
begin
   with Sender.Canvas{$ifndef F_P}^{$endif} do begin
      M:= Sender.ClientRect;
      Brush.Color:= Sender.Color;
      Fillrect(M);
      LOGPIXELSX_ := GetDeviceCaps(DC,LOGPIXELSX);
      LOGPIXELSY_ := GetDeviceCaps(DC,LOGPIXELSY);
      SelectObject(DC, Sender.Font.Handle);
      x := 0; y := 0;
      repeat
         x := x + 0.5 ; y := y + 0.5 ;
         case FUseUnit of
            suCentimeters: begin
               pt.x := round( x * LOGPIXELSX_ / 2.54);
               pt.y := round( y * LOGPIXELSY_ / 2.54); end;
            suInches: begin
               pt.x := round( x * LOGPIXELSX_ );
               pt.y := round( y * LOGPIXELSY_ ); end;
            suPixels: begin
               pt.x := round( x * 50 );
               pt.y := round( y * 50 ); end;
         end;
         if pt.x > sender.Width then break;
         if x = trunc(x) then begin
            R := MakeRect( pt.x-10, 0, pt.x+10, sender.Height);
            SetBkMode( DC, TRANSPARENT );
            if FUseUnit = suPixels then str := Int2Str(pt.x) else str := Int2Str(trunc(X));
            Windows.DrawText( DC, PChar(str), Length(str), R, DT_SINGLELINE or DT_CENTER );
         end;
         MoveTo( pt.x, Sender.Height div 2 + Offset[x = trunc(x)] );
         LineTo( pt.x, Sender.Height div 2 + 1 );
      until false;
      FDown.DrawTransparent( DC, FLeftPosition - 4, M.Top, clFuchsia);
      FUp.DrawTransparent( DC, FRightPosition - 4, M.Bottom - FUp.Height, clFuchsia);
   end;
   Wnd:= ToIntegerEvent(_data_DHandle);
   if Wnd = 0 then exit; 
   InvalidateRect(Wnd,nil,true);
   PaintLine;
end;

procedure THIRuler.SetLeftPosition;
begin
  Fold:= FLeftposition;
  if FLeftPosition = Value then exit;
  FLeftPosition := Max(FParent.Clientrect.left,Min(FParent.Clientwidth,Value));
  if FleftPosition >= FRightPosition then Swap(FleftPosition,FRightPosition);
end;

procedure THIRuler.SetRightPosition;
begin
  Fold:= FRightposition;
  if FRightPosition = Value then exit;
  FRightPosition := Max(FParent.ClientRect.left,Min(FParent.Clientwidth,Value));
  if FleftPosition >= FrightPosition then Swap(FleftPosition,FRightPosition);
end;

procedure THIRuler._var_CurPosition;
begin
   if FFlag then
      dtInteger(_Data, FLeftPosition)
   else
      dtInteger(_Data, FRightPosition);
end;

procedure THIRuler._var_ULeftPosition;
begin
   LOGPIXELSX_ := GetDeviceCaps(Control.Canvas.Handle,LOGPIXELSX);
   if LOGPIXELSX_ = 0 then exit;
   case FUseUnit of
      suCentimeters:
         dtReal(_Data, round(FLeftPosition*2.54*100/LOGPIXELSX_)/100);
      suInches:
         dtReal(_Data, round(FLeftPosition*100/LOGPIXELSX_)/100);
      suPixels:
         dtInteger(_Data, FLeftPosition);
      end;
end;

procedure THIRuler._var_URightPosition;
begin
   LOGPIXELSX_ := GetDeviceCaps(Control.Canvas.Handle,LOGPIXELSX);
   case FUseUnit of
      suCentimeters:
         dtReal(_Data, round(FRightPosition*2.54*100/LOGPIXELSX_)/100);
      suInches:
         dtReal(_Data, round(FRightPosition*100/LOGPIXELSX_)/100);
      suPixels:
         dtInteger(_Data, FRightPosition);
      end;
end;

function THIRuler.PaintLine;
var   WDC: HDC;
      Wnd: HWND;
      Pen: HPEN;
      WRect: TRect;
begin
   if Assigned(Applet)and(not AppletTerminated) then Applet.ProcessMessages;
   Wnd:= ToIntegerEvent(_data_DHandle);
   WDC:= GetDC(Wnd);
   GetClientRect(Wnd, WRect);
   Pen:= CreatePen(_prop_LineStyle,1,Color2RGB(_prop_LineColor));
   SelectObject(WDC,Pen);
   if FFlag then begin
      MoveToEx(WDC,WRect.Left + FLeftPosition, WRect.Top,nil);
      LineTo(WDC,WRect.Left + FLeftPosition, WRect.Bottom);
   end
   else begin
      MoveToEx(WDC,WRect.Left + FRightPosition, WRect.Top,nil);
      LineTo(WDC,WRect.Left + FRightPosition, WRect.Bottom);
   end;
   DeleteObject(Pen);
   ReleaseDC(Wnd,WDC);
end;

procedure THIRuler.InitRDO;
begin
   FDown:=NewDIBBitmap(9,8,pf32bit);
   with FDown{$ifndef F_P}^{$endif} do begin
      Canvas.Brush.Color := clFuchsia;
      Canvas.FillRect(MakeRect(0, 0, 9, 8));
      Canvas.Brush.Color := clBlack;
      Canvas.FillRect(MakeRect(0, 0, 9, 4));
      Canvas.FillRect(MakeRect(1, 4, 8, 5));
      Canvas.FillRect(MakeRect(2, 5, 7, 6));
      Canvas.FillRect(MakeRect(3, 6, 6, 7));
      Canvas.FillRect(MakeRect(4, 7, 5, 8));
      Canvas.Brush.Color := clGray;
      Canvas.FillRect(MakeRect(1, 1, 8, 4));
      Canvas.FillRect(MakeRect(2, 4, 7, 5));
      Canvas.FillRect(MakeRect(3, 5, 6, 6));
      Canvas.FillRect(MakeRect(4, 6, 5, 7));
      Canvas.Brush.Color := clWhite;
      Canvas.FillRect(MakeRect(1, 1, 7, 4));
      Canvas.FillRect(MakeRect(2, 4, 6, 5));
      Canvas.FillRect(MakeRect(3, 5, 5, 6));
      Canvas.Brush.Color := clSilver;
      Canvas.FillRect(MakeRect(2, 2, 7, 4));
      Canvas.FillRect(MakeRect(3, 4, 6, 5));
      Canvas.FillRect(MakeRect(4, 5, 5, 6));
   end;
end;

procedure THIRuler.InitRUP;
begin
   FUp:=NewDIBBitmap(9,8,pf32bit);
   with FUp{$ifndef F_P}^{$endif} do begin
      Canvas.Brush.Color := clFuchsia;
      Canvas.FillRect(MakeRect(0, 0, 9, 8));
      Canvas.Brush.Color := clBlack;
      Canvas.FillRect(MakeRect(0, 4, 9, 8));
      Canvas.FillRect(MakeRect(1, 3, 8, 4));
      Canvas.FillRect(MakeRect(2, 2, 7, 3));
      Canvas.FillRect(MakeRect(3, 1, 6, 2));
      Canvas.FillRect(MakeRect(4, 0, 5, 1));
      Canvas.Brush.Color := clGray;
      Canvas.FillRect(MakeRect(1, 4, 8, 7));
      Canvas.FillRect(MakeRect(2, 3, 7, 4));
      Canvas.FillRect(MakeRect(3, 2, 6, 3));
      Canvas.FillRect(MakeRect(4, 1, 5, 2));
      Canvas.Brush.Color := clWhite;
      Canvas.FillRect(MakeRect(1, 4, 7, 6));
      Canvas.FillRect(MakeRect(2, 3, 6, 4));
      Canvas.FillRect(MakeRect(3, 2, 5, 3));
      Canvas.Brush.Color := clSilver;
      Canvas.FillRect(MakeRect(2, 4, 7, 6));
      Canvas.FillRect(MakeRect(3, 3, 6, 4));
      Canvas.FillRect(MakeRect(4, 2, 5, 3));
   end;
end;

end.